@@ -1,6 +1,4 @@
 /*
- * api-session.c
- *
  * Implements the PAM session group API (pam_sm_open_session and
  * pam_sm_close_session).
  *
@@ -11,6 +9,11 @@
  * These calls aren't logged since we don't bother doing option parsing here;
  * instead, we defer that for pam_sm_setcred or don't bother doing it at all
  * for pam_sm_close_session.
+ *
+ * Copyright 2005, 2006, 2007 Russ Allbery <rra@debian.org>
+ * Copyright 2005 Andres Salomon <dilinger@debian.org>
+ * Copyright 1999, 2000 Frank Cusack <fcusack@fcusack.com>
+ * See LICENSE for licensing terms.
  */
 
 /* Get prototypes for both the authentication and session functions. */
@@ -19,14 +22,19 @@
 
 #include "config.h"
 
-#include <security/pam_appl.h>
-#include <security/pam_modules.h>
+#ifdef HAVE_SECURITY_PAM_APPL_H
+# include <security/pam_appl.h>
+# include <security/pam_modules.h>
+#elif HAVE_PAM_PAM_APPL_H
+# include <pam/pam_appl.h>
+# include <pam/pam_modules.h>
+#endif
 
 #include "internal.h"
 
 /* Store the user's credentials.  The flags are ignored. */
 int
-pam_sm_open_session(pam_handle_t *pamh, int flags, int argc,
+pam_sm_open_session(pam_handle_t *pamh, int flags UNUSED, int argc,
                     const char **argv)
 {
     return pam_sm_setcred(pamh, PAM_ESTABLISH_CRED, argc, argv);
@@ -39,8 +47,8 @@ pam_sm_open_session(pam_handle_t *pamh, int flags, int argc,
  * The flags are ignored.
  */
 int
-pam_sm_close_session(pam_handle_t *pamh, int flags, int argc,
-                     const char **argv)
+pam_sm_close_session(pam_handle_t *pamh, int flags UNUSED, int argc UNUSED,
+                     const char **argv UNUSED)
 {
-    return pam_set_data(pamh, "ctx", NULL, NULL);
+    return pam_set_data(pamh, "pam_krb5", NULL, NULL);
 }
