@@ -1,16 +1,25 @@
 /*
- * context.c
- *
  * Manage context structure.
  *
- * The context structure is the internal state maintained by the pam_krb5
+ * The context structure is the internal state maintained by the pam-krb5
  * module between calls to the various public interfaces.
+ *
+ * Copyright 2005, 2006, 2007, 2008 Russ Allbery <rra@debian.org>
+ * Copyright 2005 Andres Salomon <dilinger@debian.org>
+ * Copyright 1999, 2000 Frank Cusack <fcusack@fcusack.com>
+ * See LICENSE for licensing terms.
  */
 
 #include "config.h"
 
-#include <security/pam_appl.h>
-#include <security/pam_modules.h>
+#include <errno.h>
+#ifdef HAVE_SECURITY_PAM_APPL_H
+# include <security/pam_appl.h>
+# include <security/pam_modules.h>
+#elif HAVE_PAM_PAM_APPL_H
+# include <pam/pam_appl.h>
+# include <pam/pam_modules.h>
+#endif
 #include <stdlib.h>
 #include <string.h>
 
@@ -31,13 +40,15 @@ pamk5_context_new(struct pam_args *args)
 {
     struct context *ctx;
     int retval;
-    const char *name;
+    PAM_CONST char *name;
 
     ctx = calloc(1, sizeof(struct context));
     if (ctx == NULL) {
         retval = PAM_BUF_ERR;
         goto done;
     }
+    ctx->cache = NULL;
+    ctx->princ = NULL;
     ctx->creds = NULL;
     args->ctx = ctx;
 
@@ -90,7 +101,7 @@ pamk5_context_fetch(struct pam_args *args)
 {
     int pamret;
 
-    pamret = pam_get_data(args->pamh, "ctx", (void *) &args->ctx);
+    pamret = pam_get_data(args->pamh, "pam_krb5", (void *) &args->ctx);
     if (pamret != PAM_SUCCESS)
         args->ctx = NULL;
     return (pamret == 0 && args->ctx == NULL) ? PAM_SERVICE_ERR : pamret;
@@ -134,7 +145,8 @@ pamk5_context_free(struct context *ctx)
  * pamk5_context_free.
  */
 void
-pamk5_context_destroy(pam_handle_t *pamh, void *data, int pam_end_status)
+pamk5_context_destroy(pam_handle_t *pamh UNUSED, void *data,
+                      int pam_end_status UNUSED)
 {
     struct context *ctx = (struct context *) data;
 
